# Copyright (c)  2020  Xiaomi Corporation (authors: Fangjun Kuang)

import _kaldilm


def arpa2fst(input_arpa: str,
             output_fst: str = '',
             bos_symbol: str = '<s>',
             disambig_symbol: str = '',
             eos_symbol: str = '</s>',
             ilabel_sort: bool = True,
             keep_symbols: bool = False,
             max_arpa_warnings: int = 30,
             read_symbol_table: str = '',
             write_symbol_table: str = '',
             max_order: int = -1) -> str:
    '''Convert an ARPA file to an FST.

    This function is a wrapper of kaldi's arpa2fst and
    all the arguments have the same meaning with their counterparts in kaldi.

    Args:
      input_arpa:
        The input arpa file.
      output_fst:
        The output fst file. Note that it is a binary file.
        This function will return a text format of it.
        If it is an empty string, no output file is written.
      bos_symbol:
        Beginning of sentence symbol.
      disambig_symbol:
        Disambiguator. If provided (e.g., #0), used on input side of backoff
        links, and <s> and </s> are replaced with epsilons.
      eos_symbol:
        End of sentence symbol.
      ilabel_sort:
        Ilabel-sort the output FST.
      keep_symbols:
        Store symbol table with FST. Symbols always saved to FST if symbol
        tables are neither read or written (otherwise symbols would be lost
        entirely).
      max_arpa_warnings:
        Maximum warnings to report on ARPA parsing, 0 to disable, -1 to
        show all.
      read_symbol_table:
        Use existing symbol table.
      write_symbol_table:
        Write generated symbol table to a file.
      max_order:
        Maximum order (inclusive) in the arpa file is used to generate the final FST.
        If it is -1, all ngram data in the file are used.
        If it is 1, only unigram data are used.
        If it is 2, only ngram data up to bigram are used.

    Returns:
      Return a text format of the resulting FST with integer labels.
    '''
    s = _kaldilm.arpa2fst(input_arpa=input_arpa,
                          output_fst=output_fst,
                          bos_symbol=bos_symbol,
                          disambig_symbol=disambig_symbol,
                          eos_symbol=eos_symbol,
                          ilabel_sort=ilabel_sort,
                          keep_symbols=keep_symbols,
                          max_arpa_warnings=max_arpa_warnings,
                          read_symbol_table=read_symbol_table,
                          write_symbol_table=write_symbol_table,
                          max_order=max_order)
    return s



#!/usr/bin/env python3
#
# Copyright (c)  2020  Xiaomi Corporation (authors: Fangjun Kuang)

if __name__ == '__main__':
    import argparse

    from .arpa2fst import arpa2fst

    def _str2bool(v):
        '''
        This function is modified from
        https://stackoverflow.com/questions/15008758/parsing-boolean-values-with-argparse
        '''
        if isinstance(v, bool):
            return v
        elif v.lower() in ('yes', 'true', 't', 'y', '1'):
            return True
        elif v.lower() in ('no', 'false', 'f', 'n', '0'):
            return False
        else:
            raise argparse.ArgumentTypeError('Boolean value expected.')

    parser = argparse.ArgumentParser("Python wrapper of kaldi's arpa2fst")

    parser.add_argument('--bos-symbol',
                        help='Beginning of sentence symbol (default = "<s>")',
                        default='<s>')
    parser.add_argument('--disambig-symbol',
                        help='Disambiguator. If provided (e.g., #0), '
                        'used on input side of backoff links, '
                        'and <s> and </s> are replaced '
                        'with epsilons (default = "")',
                        default='')
    parser.add_argument('--eos-symbol',
                        help='End of sentence symbol (default = "</s>")',
                        default='</s>')
    parser.add_argument('--ilabel-sort',
                        help='Ilabel-sort the output FST (default = true)',
                        type=_str2bool,
                        default=True)
    parser.add_argument('--keep-symbols',
                        help='Store symbol table with FST. '
                        'Symbols always saved to FST if symbol'
                        'tables are neither read or written '
                        '(otherwise symbols would be lost entirely) '
                        '(default = false)',
                        type=_str2bool,
                        default=False)
    parser.add_argument('--max-arpa-warnings',
                        help='Maximum warnings to report on ARPA parsing, '
                        '0 to disable, -1 to show all (default = 30)',
                        type=int,
                        default=30)
    parser.add_argument('--read-symbol-table',
                        help='Use existing symbol table (default = "")',
                        default='')
    parser.add_argument('--write-symbol-table',
                        help='(Write generated symbol table to '
                        'a file (default = "")',
                        default='')
    parser.add_argument('--max-order',
                        help='Maximum order (inclusive) in the arpa file is '
                        'used to generate the final FST. '
                        'If it is -1, all ngram data in the file are used.'
                        'If it is 1, only unigram data are used.'
                        'If it is 2, only ngram data up to bigram are used.'
                        'Default is -1.',
                        default=-1,
                        type=int)
    parser.add_argument('input_arpa', help='input arpa filename')
    parser.add_argument('output_fst',
                        default='',
                        nargs='?',
                        help='Output fst filename. '
                        'If empty, no output file is created.')
    args = parser.parse_args()

    s = arpa2fst(input_arpa=args.input_arpa,
                 output_fst=args.output_fst,
                 bos_symbol=args.bos_symbol,
                 disambig_symbol=args.disambig_symbol,
                 eos_symbol=args.eos_symbol,
                 ilabel_sort=args.ilabel_sort,
                 keep_symbols=args.keep_symbols,
                 max_arpa_warnings=args.max_arpa_warnings,
                 read_symbol_table=args.read_symbol_table,
                 write_symbol_table=args.write_symbol_table,
                 max_order=args.max_order)
    print(s)



#!/usr/bin/env python3
#
# Copyright (c)  2020  Xiaomi Corporation (author: Fangjun Kuang)

import glob
import os
import platform
import re
import shutil
import subprocess
import sys

import setuptools
from setuptools.command.build_ext import build_ext

cur_dir = os.path.dirname(os.path.abspath(__file__))


def is_windows():
    return platform.system() == "Windows"


def cmake_extension(name, *args, **kwargs) -> setuptools.Extension:
    kwargs["language"] = "c++"
    sources = []
    return setuptools.Extension(name, sources, *args, **kwargs)


class BuildExtension(build_ext):
    def build_extension(self, ext: setuptools.extension.Extension):
        build_dir = self.build_temp
        os.makedirs(build_dir, exist_ok=True)

        os.makedirs(self.build_lib, exist_ok=True)

        cmake_args = os.environ.get("KALDILM_CMAKE_ARGS", "")
        make_args = os.environ.get("KALDILM_MAKE_ARGS", "")
        system_make_args = os.environ.get("MAKEFLAGS", "")

        if cmake_args == "":
            cmake_args = "-DCMAKE_BUILD_TYPE=Release"

        if make_args == "" and system_make_args == "":
            print("For fast compilation, run:")
            print('export KALDILM_MAKE_ARGS="-j"; python setup.py install')
            make_args = " -j4 "
            print("Setting make_args to '-j4'")

        if "PYTHON_EXECUTABLE" not in cmake_args:
            print(f"Setting PYTHON_EXECUTABLE to {sys.executable}")
            cmake_args += f" -DPYTHON_EXECUTABLE={sys.executable}"

        if not is_windows():
            ret = os.system(
                f"cd {build_dir}; cmake {cmake_args} {cur_dir}; make -j _kaldilm"
            )
            if ret != 0:
                raise Exception(
                    "\nBuild kaldilm failed. Please check the error message.\n"
                    "You can ask for help by creating an issue on GitHub.\n"
                    "\nClick:\n    https://github.com/csukuangfj/kaldilm/issues/new\n"
                )
            lib_so = glob.glob(f"{build_dir}/lib/*.so*")
            for so in lib_so:
                print(f"Copying {so} to {self.build_lib}/")
                shutil.copy(f"{so}", f"{self.build_lib}/")

            # macos
            # also need to copy *fst*.dylib
            lib_so = glob.glob(f"{build_dir}/lib/*.dylib*")
            for so in lib_so:
                print(f"Copying {so} to {self.build_lib}/")
                shutil.copy(f"{so}", f"{self.build_lib}/")
            return
        # for windows

        build_cmd = f"""
            cmake {cmake_args} -B {self.build_temp} -S {cur_dir}
            cmake --build {self.build_temp} --target _kaldilm --config Release -- -m
        """
        print(f"build command is:\n{build_cmd}")

        ret = os.system(f"cmake {cmake_args} -B {self.build_temp} -S {cur_dir}")
        if ret != 0:
            raise Exception("Failed to build kaldilm")
        ret = os.system(
            f"cmake --build {self.build_temp} --target _kaldilm --config Release -- -m"
        )
        if ret != 0:
            raise Exception("Failed to build kaldilm")

        # bin/Release/_kaldilm.cp38-win_amd64.pyd
        lib_so = glob.glob(f"{self.build_temp}/**/*.pyd", recursive=True)
        for so in lib_so:
            print(f"Copying {so} to {self.build_lib}/")
            shutil.copy(f"{so}", f"{self.build_lib}/")

        # lib/Release/{_kaldilm, openfst}.lib
        lib_so = glob.glob(f"{self.build_temp}/**/*.lib", recursive=True)
        for so in lib_so:
            print(f"Copying {so} to {self.build_lib}/")
            shutil.copy(f"{so}", f"{self.build_lib}/")


def read_long_description():
    with open("README.md", encoding="utf8") as f:
        readme = f.read()
    return readme


def get_package_version():
    with open("CMakeLists.txt") as f:
        content = f.read()

    latest_version = re.search(r"set\(kaldilm_VERSION (.*)\)", content).group(1)
    latest_version = latest_version.strip('"')
    return latest_version


package_name = "kaldilm"

setuptools.setup(
    name=package_name,
    version=get_package_version(),
    author="Fangjun Kuang",
    author_email="csukuangfj@gmail.com",
    package_dir={
        package_name: "kaldilm/python/kaldilm",
    },
    packages=[package_name],
    url="https://github.com/csukuangfj/kaldilm",
    long_description=read_long_description(),
    long_description_content_type="text/markdown",
    ext_modules=[cmake_extension("_kaldilm")],
    cmdclass={"build_ext": BuildExtension},
    zip_safe=False,
    classifiers=[
        "Programming Language :: C++",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    license="Apache licensed, as found in the LICENSE file",
)
